"""
    1Password Connect

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.2.1
    Generated by: https://openapi-generator.tech
"""


import pprint


class ItemDetails:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'sections': 'list[Section]',
        'fields': 'list[Field]'
    }

    attribute_map = {
        'sections': 'sections',
        'fields': 'fields'
    }

    def __init__(self, sections=None, fields=None):  # noqa: E501
        self._sections = None
        self._fields = None
        self.discriminator = None

        if sections is not None:
            self.sections = sections
        if fields is not None:
            self.fields = fields

    @property
    def sections(self):
        """Gets the sections of this ItemDetails.  # noqa: E501


        :return: The sections of this ItemDetails.  # noqa: E501
        :rtype: list[Section]
        """
        return self._sections

    @sections.setter
    def sections(self, sections):
        """Sets the sections of this ItemDetails.


        :param sections: The sections of this ItemDetails.  # noqa: E501
        :type: list[Section]
        """

        self._sections = sections

    @property
    def fields(self):
        """Gets the fields of this ItemDetails.  # noqa: E501


        :return: The fields of this ItemDetails.  # noqa: E501
        :rtype: list[Field]
        """
        return self._fields

    @fields.setter
    def fields(self, fields):
        """Sets the fields of this ItemDetails.


        :param fields: The fields of this ItemDetails.  # noqa: E501
        :type: list[Field]
        """

        self._fields = fields

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ItemDetails):
            return True

        return self.to_dict() != other.to_dict()
